#!/usr/bin/env python
"""
This simple example is used for the line-by-line tutorial
that comes with pygame. It is based on a 'popular' web banner.
Note there are comments here, but for the full explanation, 
follow along in the tutorial.
"""


#Import Modules
import os, pygame, operator
from copy import copy
from pygame.locals import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

def allKUPs(x): return x.type == KEYUP

#-----------------------------------------------------------------------------
def load_png(name):
    fullname = os.path.join('data', name)
    try:
        image = pygame.image.load(fullname)
	if image.get_alpha is None:
		image = image.convert()
	else:
		image = image.convert_alpha()
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    return image, image.get_rect()

#-----------------------------------------------------------------------------
def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('data', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', fullname
        raise SystemExit, message
    return sound

#-----------------------------------------------------------------------------
class SimpleSprite(pygame.sprite.Sprite):
	def __init__(self, imgName=None):
		pygame.sprite.Sprite.__init__(self) #call Sprite initializer
		if imgName:
			self.image, self.rect = load_png(imgName)
		else:
			self.image = pygame.Surface( (100,100) )
			self.image.fill( (200,0,0) )
			self.rect = self.image.get_rect()
	def update(self):
		pass
	def SetImg(self, newSurface, newRect=None):
		self.image = newSurface
		if newRect:
			self.rect  = newRect
		else:
			self.rect  = newSurface.get_rect()



#classes for our game objects
#-----------------------------------------------------------------------------
class PunchGame:
	def __init__(self, charGroup, hudGroup ):
		self.charGroup = charGroup
		self.hudGroup = hudGroup

		self.chimp = Chimp(self)

		chimpStatus = StatusBar( self.chimp )
		self.hudGroup.add( chimpStatus.GetAllSprites() )

		self.charactor = Charactor()
		self.charactor.SetEnemy( self.chimp )
		self.charGroup.add( [self.chimp, self.charactor] )

		self.difficulty = 1.0
		self.listenKeys = {}


	def OnMonkeyDeath(self, oldChimp):
		self.difficulty = self.difficulty + 0.25

    		self.chimp.kill()
		self.chimp = Chimp(self, self.difficulty)

		chimpStatus = StatusBar( self.chimp )
		self.hudGroup.empty( )
		self.hudGroup.add( chimpStatus.GetAllSprites() )

		self.charactor.SetEnemy( self.chimp )
		self.charGroup.add( self.chimp )

	
	def SignalKey( self, event, remainingEvents ):
		if self.listenKeys.has_key( event.key ):
			self.listenKeys[event.key]()
		else:
			self.charactor.SignalKey( event, remainingEvents )
		

#-----------------------------------------------------------------------------
class StatusBar(pygame.sprite.Sprite):
	"""status bar to track health of the monkey"""
	def __init__(self, charactor):
		pygame.sprite.Sprite.__init__(self) #call Sprite initializer
		self.image, self.rect = load_png('monkeybar.png')
		self.amtSprite = SimpleSprite()
		self.charactor = charactor
		self.charactor.SetStatusBar( self )
		self.rect.move_ip( 50, 10 )
		self.amt = 100
		self.UpdateImg()

	def update(self):
		pass

	def GetAllSprites(self):
		return (self, self.amtSprite)


	def SetAmt(self, newAmt):
		self.amt = newAmt
		self.UpdateImg()
	
	def UpdateImg(self):
		self.amtSprite.rect.top = self.rect.top + 24
		self.amtSprite.rect.left = self.rect.left + 5
		img, rect = load_png('barfill.png')
		fullHeight = rect.height
		fullWidth  = rect.width

		width = fullWidth * (self.amt/100.00)
		if width < 0:
			width = 0
		height = fullHeight
		self.amtSprite.image = \
		                  pygame.transform.scale( img, (width,height) )

		groupList = self.groups()
		self.amtSprite.add( groupList )
		#print self.amtSprite
		
#-----------------------------------------------------------------------------
class BoundingBox:
	def __init__(self):
		self.top = None;
		self.right = None;
		self.bottom = None;
		self.left = None;

		self.topSprite = None;
		self.rightSprite = None;
		self.bottomSprite = None;
		self.leftSprite = None;

#-----------------------------------------------------------------------------
class Charactor(pygame.sprite.Sprite):
    """moves a charactor on the screen, following the mouse"""
    def __init__(self):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
        self.image, self.rect = load_png('charactor.png')
	self.rightFacingImg = self.image
	self.leftFacingImg = pygame.transform.flip( self.image, 1, 0)

	self.rect.midbottom = ( 600, 300 )
	self.rect.inflate_ip(-2,-2)
        self.area = screen.get_rect().inflate(50,50)
	self.oldPos = self.rect

	self.listenKeys = { K_UP: self.MoveUp,
	                    K_DOWN: self.MoveDown,
	                    K_LEFT: self.MoveLeft,
	                    K_RIGHT: self.MoveRight,
	                    K_SPACE: self.Punch
	                  }

	self.offKeys = { K_UP: self.StopUp,
	                    K_DOWN: self.StopDown,
	                    K_LEFT: self.StopLeft,
	                    K_RIGHT: self.StopRight,
	                    K_SPACE: self.UnPunch
	                  }

	self.moveState = [0,0]
	self.speed     = [10,10]
	self.jumping = 0
	self.ducking = 0
	self.falling = 1
	self.snd_jump = load_sound('jump.wav')
	self.snd_land = load_sound('land.wav')
	self.snd_htop = load_sound('top.wav')

    #--------------------------------------------------------------------------
    def SetEnemy(self, enemy):
	self.enemy = enemy

    #--------------------------------------------------------------------------
    def update(self):
        "move the charactor based on the mouse position"
	self.Move()
	if self.jumping:
		self.JumpStep()
	if self.falling:
		self.FallStep()
	if self.moveState[0] < 0:
		self.image = self.rightFacingImg
	if self.moveState[0] > 0:
		self.image = self.leftFacingImg

    #--------------------------------------------------------------------------
    def Move( self ):
	#the change in position is the direction they're moving times the
	# speed they're going
    	change = map(operator.mul, self.moveState, self.speed )

	if change == [0,0]:
		return

	#print change

        self.oldPos = self.rect.move( (0,0) )
        self.rect.move_ip( change )
        newPos = self.rect

	#the position is limited by the maximum allowed area
	global Solids
	limiter = self.Blocked( newPos, Solids )
	if limiter.top is not None:
		self.rect.top = limiter.top
		self.BumpTop( limiter.topSprite )
	if limiter.right is not None:
		self.rect.right = limiter.right
	if limiter.bottom is not None:
		self.rect.bottom = limiter.bottom
		self.BumpBottom( limiter.bottomSprite )
	if limiter.left is not None:
		self.rect.left = limiter.left

	
    #--------------------------------------------------------------------------
    def Blocked( self, newPos, spriteGroup ):
	limiter = BoundingBox()
	oldPos = self.oldPos

	if newPos.top    <= self.area.top:
		limiter.top = self.area.top
	if newPos.right  >= self.area.right:
		limiter.right = self.area.right
	if newPos.bottom >= self.area.bottom:
		limiter.bottom = self.area.bottom
	if newPos.left   <= self.area.left:
		limiter.left = self.area.left

	colliders = pygame.sprite.spritecollide( self, spriteGroup, 0)


	if len(colliders)  is not 0:
		for sprite in colliders:

			if ( limiter.top is None \
			   or sprite.rect.bottom > limiter.top ) \
			   and newPos.top      < sprite.rect.bottom \
			   and oldPos.top        >= sprite.rect.bottom:
				limiter.top        = sprite.rect.bottom
				limiter.topSprite  = sprite

			if ( limiter.right is None  \
			   or sprite.rect.left  < limiter.right ) \
			   and newPos.right    > sprite.rect.left \
			   and oldPos.right      <= sprite.rect.left:
				limiter.right        = sprite.rect.left
				limiter.rightSprite  = sprite

			if ( limiter.bottom is None \
			   or sprite.rect.top < limiter.bottom ) \
			   and newPos.bottom > sprite.rect.top \
			   and oldPos.bottom   <= sprite.rect.top:
				limiter.bottom       = sprite.rect.top
				limiter.bottomSprite = sprite

			if ( limiter.left is None \
			   or sprite.rect.right  > limiter.left ) \
			   and newPos.left   < sprite.rect.right \
			   and oldPos.left     >= sprite.rect.right:
				limiter.left       = sprite.rect.right
				limiter.leftSprite = sprite

	return limiter


    #--------------------------------------------------------------------------
    def MoveLeft( self, remainingEvents ):
	self.moveState[0] = -1

    #--------------------------------------------------------------------------
    def MoveRight( self, remainingEvents ):
	self.moveState[0] = 1

    #--------------------------------------------------------------------------
    def MoveUp( self, remainingEvents ):
	if not self.jumping:
		self.Jump()

    #--------------------------------------------------------------------------
    def Jump( self ):
	self.snd_jump.play()
	self.jumping = 1
	self.falling = 0
	self.moveState[1] = self.moveState[1] -5

    #--------------------------------------------------------------------------
    def JumpStep( self ):
    	if self.moveState[1] > 0:
		self.jumping = 0
		self.falling = 1
		return
	self.moveState[1] = self.moveState[1] + 0.5

    #--------------------------------------------------------------------------
    def FallStep( self ):
	if self.moveState[1] < 4:
		self.moveState[1] = self.moveState[1] + 0.2

    	change = map(operator.mul, self.moveState, self.speed )
	if change == [0,0]:
		return

        newPos = self.rect.move( change )
	enemyGroup = pygame.sprite.Group(self.enemy)
	lim = self.Blocked( newPos, enemyGroup )
	enemyGroup.empty()
	if lim.bottomSprite is not None:
		self.Punch( self.enemy )
		self.moveState[1] = -1

    #--------------------------------------------------------------------------
    def BumpBottom(self, bottomSprite ):
    	if self.moveState[1] > 2:
		self.snd_land.play()
	#self.falling = 0
	self.moveState[1] = 0

    #--------------------------------------------------------------------------
    def BumpTop(self, topSprite ):
	self.snd_htop.play()
	self.jumping = 0
	self.falling = 1
	self.moveState[1] = 0



    #--------------------------------------------------------------------------
    def MoveDown( self, remainingEvents ):
	if not self.jumping:
		self.ducking = 1

    #--------------------------------------------------------------------------
    def StopLeft( self, remainingEvents ):
	if self.moveState[0] is -1:
		self.moveState[0] = 0
    
    #--------------------------------------------------------------------------
    def StopRight( self, remainingEvents ):
	if self.moveState[0] is 1:
		self.moveState[0] = 0
    
    #--------------------------------------------------------------------------
    def StopUp( self, remainingEvents ):
	return
	if self.moveState[1] is -1:
		self.moveState[1] = 0
    
    #--------------------------------------------------------------------------
    def StopDown( self, remainingEvents ):
	self.ducking = 0
	if self.moveState[1] is 1:
		self.moveState[1] = 0
    
    #--------------------------------------------------------------------------
    def Punch(self, remainingEvents):
	self.enemy.punched(damage=30)
	return


    #--------------------------------------------------------------------------
    def UnPunch(self, remainingEvents ):
        "called to pull the charactor back"
	return

    #--------------------------------------------------------------------------
    def SignalKey( self, event, remainingEvents ):
		if self.listenKeys.has_key( event.key ) \
		  and event.type is KEYDOWN:
			self.listenKeys[event.key]( remainingEvents )
		elif self.offKeys.has_key( event.key ) \
		  and event.type is KEYUP:
			self.offKeys[event.key]( remainingEvents  )



#-----------------------------------------------------------------------------
class Chimp(pygame.sprite.Sprite):
    """moves a monkey critter across the screen. it can spin the
       monkey when it is punched."""
    def __init__(self, game, difficulty=1):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.image, self.rect = load_png('chimp.png')
        self.original = self.image
        screen = pygame.display.get_surface()
        self.area = screen.get_rect()
        self.rect.topleft = 10, 100
	self.facing = 0
        self.move = 6*difficulty
        self.moveY = 6*difficulty
        self.dizzy = 0
	self.statusBar = None
	self.health = 100
	self.game = game

    def SetStatusBar(self, bar):
    	self.statusBar = bar

    def update(self):
        "walk or spin, depending on the monkeys state"
        if self.dizzy:
            self._spin()
        else:
            self._walk()

    def _walk(self):
	"move the monkey across the screen, and turn at the ends"
	if self.rect.left < self.area.left or \
	   self.rect.right > self.area.right:
		if self.facing:
			self.facing = 0
		else:
			self.facing = 1
		self.move = -1 * self.move
		self.image = pygame.transform.flip(self.image, 1, 0)
	if self.rect.top < self.area.top or \
	   self.rect.bottom > self.area.bottom :
		self.moveY = -1 * self.moveY
	newpos = self.rect.move((self.move, self.moveY))
	self.rect = newpos

    def _spin(self):
        "spin the monkey image"
        center = self.rect.center
        self.dizzy = self.dizzy + 12
        if self.dizzy >= 360:
            self.dizzy = 0
            self.image = self.original
        else:
            rotate = pygame.transform.rotate
            self.image = rotate(self.original, self.dizzy)
        self.rect = self.image.get_rect()
        self.rect.center = center

    def Die(self):
	self.game.OnMonkeyDeath(self)


    def punched(self, damage=30):
        "this will cause the monkey to start spinning"
        if not self.dizzy:
            self.dizzy = 1
	    self.health = self.health - damage
	    if self.health < 0:
		    self.health = 0
		    self.Die()
	    if self.statusBar:
		    self.statusBar.SetAmt(self.health)




#-----------------------------------------------------------------------------
def main():
	"""this function is called when the program starts.
	   it initializes everything it needs, then runs in
	   a loop until the function returns."""
#Initialize Everything
	global screen
	screen = None
	pygame.init()
	screen = pygame.display.set_mode((868, 600))
	pygame.display.set_caption('Monkey Simulator 2000!')
	#pygame.mouse.set_visible(0)

#Create The Backgound
	global bgsurface
	bgsurface, garbage = load_png('backgr.png')

#Display The Background
	screen.blit(bgsurface, (0, 0))
	pygame.display.flip()

#Prepare Game Objects
	clock = pygame.time.Clock()

	global Solids
	Solids = pygame.sprite.RenderUpdates()
	allsprites = pygame.sprite.RenderUpdates()
	hudSprites = pygame.sprite.RenderUpdates()
	Solids = pygame.sprite.RenderUpdates()

#put in the two platforms to walk on
	plat1 = SimpleSprite('plat1.png')
	plat1.rect.move_ip( 100, 500 )
	plat1.rect.inflate_ip( -12, -6 )
	Solids.add( plat1 )
	plat1 = SimpleSprite('plat2.png')
	plat1.rect.move_ip( 400, 200 )
	plat1.rect.inflate_ip( -12, -6 )
	Solids.add( plat1 )

	game  = PunchGame(allsprites, hudSprites)

#Main Loop
	oldfps = 0
	while 1:
		clock.tick(40)
		newfps = int(clock.get_fps())
		if newfps is not oldfps:
			print "fps: ", clock.get_fps()
			oldfps = newfps

	#Handle Input Events
		oldEvents = []
		remainingEvents = pygame.event.get()
		for event in remainingEvents:
			oldEvents.append( remainingEvents.pop(0) )
			upKeys = filter( allKUPs, remainingEvents )
			#print event
			if event.type == QUIT:
				return
			elif event.type == KEYDOWN and event.key == K_ESCAPE:
				return
			elif event.type == KEYDOWN or event.type == KEYUP:
				if len(upKeys) > 0:
					print upKeys 
				game.SignalKey( event, upKeys )

	#Draw Everything

		allsprites.clear( screen, bgsurface )
		hudSprites.clear( screen, bgsurface )
		Solids.clear( screen, bgsurface )

		allsprites.update()
		hudSprites.update()

		changedRects4 = []

		changedRects1 = hudSprites.draw(screen)
		changedRects3 = Solids.draw(screen)
		changedRects2 = allsprites.draw(screen)
		pygame.display.update( changedRects1 \
		                       + changedRects2 \
		                       + changedRects3 \
		                     )

#Game Over


#this calls the 'main' function when this script is executed
if __name__ == '__main__': main()
